/****************************************************************
 * fpxoutfbm.c
 ****************************************************************/

/******
  Copyright (C) 1995 by Klaus Ehrenfried. 

  Permission to use, copy, modify, and distribute this software
  is hereby granted, provided that the above copyright notice appears 
  in all copies and that the software is available to all free of charge. 
  The author disclaims all warranties with regard to this software, 
  including all implied warranties of merchant-ability and fitness. 
  The code is simply distributed as it is.
*******/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "fbm.h"
#include "fpxpro.h"
#include <stdlib.h>

#define MAX_LEN 512

int write_bitmap();
int alloc_fbm();
int free_fbm();

int out_fbm
(
 unsigned char *image_buffer, 
 unsigned char *color_table,
 int width, 
 int height, 
 int outtype,
 char *name_base,
 char *name_ext,
 int digits,
 int num,
 int compress_flag
 )
{
  FILE *fopen(), *fpout;
  struct stat  statbuf; 
  FBM image;	/* Images */
  char out_name[MAX_LEN];
  char digiter[MAX_LEN];
  char befehl[MAX_LEN + 20];
  int m,n,i, len;

  if (name_ext == NULL)
    len=strlen(name_base)+digits+1;
  else
    len=strlen(name_base)+strlen(name_ext)+digits+1;

  if (len > MAX_LEN)
    {
      fprintf(stderr,"Image %d: File name too long!\n",num);
      return(0);
    }

  /* Now build header for output bit map */

  image.hdr.cols=width;
  image.hdr.rows=height;
  image.hdr.planes=1;
  image.hdr.bits=8;
  image.hdr.physbits=8;
  image.hdr.rowlen=image.hdr.cols;
  image.hdr.plnlen=image.hdr.cols * image.hdr.rows;
  image.hdr.clrlen=3 * MAXCOLORS;

  image.hdr.aspect=1.2;
  image.hdr.title[0]='\0';
  image.hdr.credits[0]='\0';

  image.cm=NULL;
  image.bm=NULL;

  m=num;
  for (i=(digits-1); i >= 0; i--)
    {
      n=m % 10;
      switch (n)
	{
	case 0: digiter[i] = '0'; break;
	case 1: digiter[i] = '1'; break;
	case 2: digiter[i] = '2'; break;
	case 3: digiter[i] = '3'; break;
	case 4: digiter[i] = '4'; break;
	case 5: digiter[i] = '5'; break;
	case 6: digiter[i] = '6'; break;
	case 7: digiter[i] = '7'; break;
	case 8: digiter[i] = '8'; break;
	case 9: digiter[i] = '9'; break;
	default: break;
	}
      m = (m - n) / 10;
    }
  digiter[digits] = '\0';

  if (name_ext == NULL)
    sprintf(out_name,"%s.%s",name_base,digiter);
  else
    sprintf(out_name,"%s%s.%s",name_base,digiter,name_ext);

  if (stat(out_name, &statbuf) == 0)	/* exists already */
    {
      fprintf(stderr,
	      "Error: file '%s' already exists -- image %d not written\n",
	      out_name,num);
      return(0);
    }

  if ((fpout=fopen(out_name, "wb")) == NULL)
  {
    fprintf(stderr," Error opening '%s'\n",out_name);
    return(0);
  }

  fprintf(stdout,"Write '%s'\n",out_name);

  if (alloc_fbm(&image) == 0)
  {
    fprintf(stderr,"Error alloc_fbm\n");
    return(0);
  }

  memcpy(image.cm, color_table, image.hdr.clrlen);
  memcpy(image.bm, image_buffer, image.hdr.plnlen);

  if (!write_bitmap (&image, fpout, outtype))
    {
      fprintf(stderr,"Error write_bitmap\n");
      fclose(fpout);
      free_fbm(&image);
      return(0);
    }

  fclose(fpout);
  free_fbm(&image);

  if (compress_flag == 0) return(1);

  if (compress_flag == 1)
    sprintf(befehl,"compress %s", out_name);
  else
    sprintf(befehl,"gzip %s", out_name);

  fprintf(stdout," .... %s\n",befehl);

  if (system(befehl) != 0)
  {
    fprintf(stderr," Error compressing '%s'\n",out_name);
    return(1);
  }

  return(1);
}
